#!/bin/bash

# WARNING: deletes and recreates:
#  link vec0
#  link ves0
#  namespace lab500ms
#
# creates temp dir with prefix veth-netem-issue in $TMPDIR or /tmp.

RTT=500ms
LDELAY=250ms
NETNS="lab$RTT"

devl=vec0
devr=ves0

TDIR=$(mktemp -d -t veth-netem-issue.XXXXXXXXXX) || exit 1
[ -d "$TDIR" ] || exit 1

# create netns and enable loopback, if missing
ip netns add $NETNS 2>/dev/null
ip netns exec $NETNS ip link set dev lo up

# create VETH pair
ip link del dev $devl 2>&1
ip link add $devl mtu 1500 type veth peer name $devr 2>/dev/null && {
	ip link set $devr netns $NETNS
}

# setup local side
ip link set $devl up
ip addr add 192.168.233.1/30 dev $devl

# setup netns side
ip netns exec $NETNS ip link set dev $devr up
ip netns exec $NETNS ip addr add 192.168.233.2/30 dev $devr

# Now add half-RTT delay to TX queue of both veth sides
tc qdisc replace dev $devl root netem delay "$LDELAY"
ip netns exec $NETNS tc qdisc replace dev $devr root netem delay "$LDELAY"

# preload arp to avoid [possible] ARP resolution delay on first ping, just for clarity
ping -c 1 192.168.233.2 >/dev/null 2>&1

# start two tcpdumps, limited to 10 packets
echo "starting tcpdump in background, output to $TDIR/vec0.pcap and $TDIR/ves0.pcap"
tcpdump -i vec0 -n -p -w "$TDIR/vec0.pcap" -c 10 net 192.168.233.0/30 and icmp >/dev/null 2>&1 &
ip netns exec $NETNS tcpdump -i ves0 -n -p -w "$TDIR/ves0.pcap" -c 10 net 192.168.233.0/30 and icmp >/dev/null 2>&1 &

sleep 1

echo
echo "ping shows 500ms RTT as expected"
ping -c 6 192.168.233.2

echo
echo "but the tcpdumps show incorrect receive timestamps: "
echo "the timestamps look more like packet creation time than packet-seen-at-capture-interface time"
echo 
echo "local side:"
tcpdump -n -v -r "$TDIR/vec0.pcap"
echo
echo "remote side:"
tcpdump -n -v -r "$TDIR/ves0.pcap"
echo
echo "pcap files preserved in $TDIR"
